<?php


namespace IPS\chatbox\modules\admin\chatbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
			
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# Build Form
		$form = new \IPS\Helpers\Form;	
		$form->addTab( 'chatbox_tab_general' );
		$form->addHeader( 'chatbox_tab_general' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_report', \IPS\Settings::i()->chatbox_report, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_ignore', \IPS\Settings::i()->chatbox_ignore, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_sound_disable', \IPS\Settings::i()->chatbox_sound_disable, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_noEmoji', \IPS\Settings::i()->chatbox_noEmoji, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_force_scroll_down', \IPS\Settings::i()->chatbox_force_scroll_down, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_sendBtn', \IPS\Settings::i()->chatbox_sendBtn, FALSE, array() ) );

		$form->addHeader( 'chatbox_tab_tabCount' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_tabCount', \IPS\Settings::i()->chatbox_tabCount, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Node( 'chatbox_tabCount_rooms', \IPS\Settings::i()->chatbox_tabCount_rooms != 0 ? explode(",", \IPS\Settings::i()->chatbox_tabCount_rooms) : 0, FALSE, array(
			'class' 			=> 'IPS\chatbox\Rooms\Room', 
			'multiple' 			=> TRUE,
			'zeroVal' 			=> 'all',
		), NULL, NULL, NULL, 'chatbox_tabCount_rooms' ) );
		
		$form->addHeader( 'chatbox_tab_time' );
		$form->add( new \IPS\Helpers\Form\Text( 'chatbox_time_format', \IPS\Settings::i()->chatbox_time_format, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_time_moment', \IPS\Settings::i()->chatbox_time_moment, FALSE ) );

		$form->addTab( 'chatbox_tab_bottom' );
		$form->addHeader( 'chatbox_header_bar_items' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_room', \IPS\Settings::i()->chatbox_bar_room, FALSE, array(
			'togglesOn'	=> array( 'chatbox_bar_show_rooms' )
		) ) );
		
		$form->add( new \IPS\Helpers\Form\Node( 'chatbox_bar_show_rooms', explode(",", \IPS\Settings::i()->chatbox_bar_show_rooms), FALSE, array(
			'class' 			=> 'IPS\chatbox\Rooms\Room', 
			'multiple' 			=> TRUE
		), NULL, NULL, NULL, 'chatbox_bar_show_rooms' ) );		
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_club', \IPS\Settings::i()->chatbox_bar_club ) );
		
		$form->add( new \IPS\Helpers\Form\Custom( 'chatbox_bar_setup_con', NULL, FALSE, array( 'getHtml' => function( $element )
		{
			return \IPS\Member::loggedIn()->language()->addToStack( 'chatbox_bar_setup_con_help' );
		} ) ) );
			
		$form->addHeader( 'chatbox_header_bar' );
		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_bar_pos', \IPS\Settings::i()->chatbox_bar_pos, FALSE, array( 'options' => array( 'right' => 'cb_bar_right', 'left' => 'cb_bar_left' ) ), NULL, NULL, NULL, 'chatbox_bar_pos' ) );
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'chatbox_bar_mini_devices', isset( \IPS\Settings::i()->chatbox_bar_mini_devices ) ? explode(',', \IPS\Settings::i()->chatbox_bar_mini_devices) : array( 'Phone', 'Tablet', 'Desktop' ), FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'chatbox_bar_mini_devices' ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_hide_mobile', \IPS\Settings::i()->chatbox_bar_hide_mobile, FALSE, array( 'togglesOff' => array( 'chatbox_bar_full' ) ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_full', \IPS\Settings::i()->chatbox_bar_full, FALSE, array(), NULL, NULL, NULL, 'chatbox_bar_full' ) );
		$form->add( new \IPS\Helpers\Form\TextArea( 'chatbox_bar_expages', \IPS\Settings::i()->chatbox_bar_expages, FALSE, array( 'rows' => 8 ) ) );
		
		$form->addTab( 'chatbox_tab_advanced' );
		if ( ! \IPS\CIC2 )
		{
			$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_cache', \IPS\Settings::i()->chatbox_cache, FALSE, array() ) );
		}		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_no_disconnect', \IPS\Settings::i()->chatbox_no_disconnect, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_upload_file_size', \IPS\Settings::i()->chatbox_upload_file_size, FALSE, array('min' => 0.1, 'max' => 50) ) );
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			unset( $values['chatbox_bar_setup_con'] );
			$form->saveAsSettings( $values );
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=chatbox&module=chatbox&controller=settings' ), 'saved' );
		}
		
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=settings" ), 'chatbox_settings' );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('chatbox_settings');	
		\IPS\Output::i()->output .= $form;
	}
	
}